@@ -4,7 +4,7 @@ from collections.abc import Generator
 from typing import Any, TypeVar
 
 from .context import InjectionContext, SyncInjectionContext
-from .providers import Provider
+from .providers import Provider, Singleton
 
 _T = TypeVar("_T")
 _Providers = dict[type[_T], list[Provider[_T]]]
@@ -62,10 +62,14 @@ class Container:
         return provider
 
     def context(self) -> InjectionContext:
-        return InjectionContext(container=self)
+        return InjectionContext(
+            container=self,
+        )
 
     def sync_context(self) -> SyncInjectionContext:
-        return SyncInjectionContext(container=self)
+        return SyncInjectionContext(
+            container=self,
+        )
 
     @contextlib.contextmanager
     def override(
@@ -75,3 +79,9 @@ class Container:
         self._overrides[provider.type].append(provider)
         yield
         self._overrides[provider.type].remove(provider)
+
+    async def aclose(self) -> None:
+        for _, providers in self.providers.items():
+            for provider in providers:
+                if isinstance(provider, Singleton):
+                    await provider.aclose()
