@@ -5,10 +5,12 @@ import contextvars
 import inspect
 import typing
 from collections.abc import Callable, Coroutine, Iterable
+from contextlib import AsyncExitStack, ExitStack
 from contextvars import ContextVar
 from types import TracebackType
-from typing import TYPE_CHECKING, Any, Protocol, TypeVar, Union
+from typing import TYPE_CHECKING, Any, Generic, Protocol, TypeVar, Union
 
+from . import utils
 from .providers import Dependency
 
 if TYPE_CHECKING:
@@ -21,35 +23,46 @@ context_var: ContextVar[_AnyCtx] = ContextVar("aioinject_context")
 
 container_var: ContextVar[Container] = ContextVar("aioinject_container")
 
-
 TypeAndImpl = tuple[type[_T], _T | None]
+TExitStack = TypeVar("TExitStack")
 
 
-class DiCache(Protocol):
+class DICache(Protocol):
     def __setitem__(self, key: TypeAndImpl[_T], value: _T) -> None:
-        ...
+        ...  # pragma: no cover
 
     def __getitem__(self, item: TypeAndImpl[_T]) -> _T:
-        ...
+        ...  # pragma: no cover
 
     def __contains__(self, item: TypeAndImpl) -> bool:
-        ...
+        ...  # pragma: no cover
 
 
-class _BaseInjectionContext:
+class _BaseInjectionContext(Generic[TExitStack]):
     _token: contextvars.Token | None
+    _exit_stack_type: type[TExitStack]
 
-    def __init__(self, container: Container) -> None:
+    def __init__(
+        self,
+        container: Container,
+    ) -> None:
         self.container = container
-        self.cache: DiCache = {}
+        self.exit_stack = self._exit_stack_type()
+        self.cache: DICache = {}
         self._token = None
 
+    def __class_getitem__(
+        cls,
+        item: type[TExitStack],
+    ) -> _BaseInjectionContext[TExitStack]:
+        return type(  # type: ignore[return-value]
+            f"_BaseInjectionContext[{item.__class__.__name__}]",
+            (cls,),
+            {"_exit_stack_type": item},
+        )
 
-class InjectionContext(_BaseInjectionContext):
-    def __init__(self, container: Container) -> None:
-        super().__init__(container=container)
-        self.exit_stack = contextlib.AsyncExitStack()
 
+class InjectionContext(_BaseInjectionContext[AsyncExitStack]):
     async def resolve(
         self,
         type_: type[_T],
@@ -59,6 +72,7 @@ class InjectionContext(_BaseInjectionContext):
     ) -> _T:
         if use_cache and (type_, impl) in self.cache:
             return self.cache[type_, impl]
+
         provider = self.container.get_provider(type_, impl)
         dependencies = {
             dep.name: await self.resolve(
@@ -69,13 +83,11 @@ class InjectionContext(_BaseInjectionContext):
             for dep in provider.dependencies
         }
 
-        resolved = await provider.provide(**dependencies)
-        if isinstance(resolved, contextlib.ContextDecorator):
-            resolved = self.exit_stack.enter_context(resolved)  # type: ignore[arg-type]
-
-        if isinstance(resolved, contextlib.AsyncContextDecorator):
-            resolved = await self.exit_stack.enter_async_context(resolved)  # type: ignore[arg-type]
-
+        resolved = utils.enter_context_maybe(
+            resolved=await provider.provide(**dependencies),
+            stack=self.exit_stack,
+        )
+        resolved = await utils.await_maybe(resolved)
         if use_cache:
             self.cache[type_, impl] = resolved
         return resolved
@@ -88,7 +100,7 @@ class InjectionContext(_BaseInjectionContext):
         *args: Any,
         **kwargs: Any,
     ) -> _T:
-        ...
+        ...  # pragma: no cover
 
     @typing.overload
     async def execute(
@@ -98,7 +110,7 @@ class InjectionContext(_BaseInjectionContext):
         *args: Any,
         **kwargs: Any,
     ) -> _T:
-        ...
+        ...  # pragma: no cover
 
     async def execute(
         self,
@@ -134,11 +146,7 @@ class InjectionContext(_BaseInjectionContext):
         await self.exit_stack.__aexit__(exc_type, exc_val, exc_tb)
 
 
-class SyncInjectionContext(_BaseInjectionContext):
-    def __init__(self, container: Container) -> None:
-        super().__init__(container=container)
-        self.exit_stack = contextlib.ExitStack()
-
+class SyncInjectionContext(_BaseInjectionContext[ExitStack]):
     def resolve(
         self,
         type_: type[_T],
@@ -162,7 +170,6 @@ class SyncInjectionContext(_BaseInjectionContext):
         resolved = provider.provide_sync(**dependencies)
         if isinstance(resolved, contextlib.ContextDecorator):
             resolved = self.exit_stack.enter_context(resolved)  # type: ignore[arg-type]
-
         if use_cache:
             self.cache[type_, impl] = resolved
         return resolved
