@@ -1,4 +1,5 @@
-from typing import Optional
+from collections.abc import Callable
+from typing import ParamSpec, TypeVar
 
 from fastapi import Request
 from starlette.middleware.base import (
@@ -13,14 +14,16 @@ import aioinject
 from aioinject import Container, utils
 from aioinject.decorators import InjectMethod
 
+_T = TypeVar("_T")
+_P = ParamSpec("_P")
 
-def inject(function):
+
+def inject(function: Callable[_P, _T]) -> Callable[_P, _T]:
     wrapper = aioinject.decorators.inject(
         function,
         inject_method=InjectMethod.context,
     )
-    wrapper = utils.clear_wrapper(wrapper)
-    return wrapper
+    return utils.clear_wrapper(wrapper)
 
 
 class InjectMiddleware(BaseHTTPMiddleware):
@@ -28,13 +31,15 @@ class InjectMiddleware(BaseHTTPMiddleware):
         self,
         app: ASGIApp,
         container: Container,
-        dispatch: Optional[DispatchFunction] = None,
+        dispatch: DispatchFunction | None = None,
     ) -> None:
         super().__init__(app, dispatch)
         self.container = container
 
     async def dispatch(
-        self, request: Request, call_next: RequestResponseEndpoint
+        self,
+        request: Request,
+        call_next: RequestResponseEndpoint,
     ) -> Response:
         async with self.container.context():
             return await call_next(request)
